function stats = AVarReMeDI2(Y,ts,pars)
%%
% avar = AVarReMeDI2(y,ts)
% Estimate the asymptotic variance of the ReMeDI estimators of the  
% autocovariances of microstructure noise
% 
% Inputs: Y --- the observed (noisy) price
%             ts --- the transaction times
%             pars --- tuning parameters
% Outputs: avar --- the estimated asympotic variance
% References: 
% Li, Z. Merrick and Oliver Linton (2020):
% "A ReMeDI for microstructure noise."
% 
% Authors : Z. Merrick Li and Oliver Linton
% 
%%
%% extract the tuning parameters
p_n = pars.phi_n;
k_n = pars.kn;
i_n = pars.in;
l = pars.l;% the order of autocovariances, e.g.,
% l = 0 corresponds to the variance of noise
n = length(ts);
% normalize the transaction times
ts = ts-ts(1);
ts = ts/ts(n);
%% calculate the obv time based statistics 
diff1 = ts(2:n) - ts(1:n-1);
if size(diff1,2)>size(diff1,1)
        diff1=diff1';
end
diffknT = ts(1+k_n:n) - ts(1:n-k_n);
if size(diffknT,2)>size(diffknT,1)
        diffknT=diffknT';
end
dn1 = k_n*diff1(2+k_n:n-k_n-1) ...
    - diffknT(k_n+3:n-k_n);
dn2 = dn1./(max(p_n,diffknT(1:n-2-2*k_n))); 
dn = dn2.^2;
%% take the kn, 2kn and 3kn differences of log-prices
diffknY = Y(1:n-k_n) - Y(k_n+1:n);
diff2knY = Y(2*k_n+1:n) - Y(1:n-2*k_n);
diff3knY = Y(3*k_n+1:n) - Y(1:n-3*k_n);
%% ReMeDI(j)
%Rj1 = Y(1+2*k_n+l:n-k_n) - Y(1+3*k_n+l:n);
%Rj2 = Y(1+2*k_n:n-l-k_n)-Y(1:n-3*k_n-l);
Rj = sum(diffknY(1+2*k_n+l:n-k_n).*...
    diff2knY(1:n-l-3*k_n));
%% U(1)
U1 = sum(dn);
%% U(2)
%R21 = Y(3+5*k_n+l:n-k_n) - Y(3+6*k_n+l:n);
%R22 = Y(3+5*k_n:n-l-k_n)-Y(3+3*k_n:n-3*k_n-l);
U2 = sum(diff2knY(3+3*k_n:n-l-3*k_n).*...
    diffknY(3+5*k_n+l:n-k_n)...
    .*dn(1:n-2-6*k_n-l));
%% U(3)
%R311 = Y(3+5*k_n+l:n-l-5*k_n)-Y(3+6*k_n+l:n-4*k_n-*l);
%R312 = Y(3+5*k_n:n-5*k_n-2*l)-Y(3+3*k_n:n-7*k_n-2*l);
%R321 = Y(3+9*k_n+2*l:n-k_n)-Y(3+10*k_n+2*l:n);
%R322 = Y(3+9*k_n+l:n-k_n-l) - Y(3+7*k_n+l:n-3*k_n-l);
U3 = sum(diffknY(3+5*k_n+l:n-l-5*k_n).*...
    diffknY(3+9*k_n+2*l:n-k_n).*...
    diff2knY(3+3*k_n:n-7*k_n-2*l).*...
    diff2knY(3+7*k_n+l:n-3*k_n-l).*...
dn(1:n-10*k_n-2*l-2));
%% U(4)
%R411 = Y(1+2*k_n+l:n-l-5*k_n)-Y(1+3*k_n+l:n-4*k_n-l);
%R412 = Y(1+2*k_n:n-5*k_n-2*l)-Y(1:n-7*k_n-2*l);
%R421 = Y(1+6*k_n+2*l:n-k_n)-Y(1+7*k_n+2*l:n);
%R422 = Y(1+6*k_n+l:n-k_n-l) - Y(1+4*k_n+l:n-3*k_n-l);
U4 = -sum(diffknY(1+2*k_n+l:n-l-5*k_n).*...
    diffknY(1+6*k_n+2*l:n-k_n).*...
    diff2knY(1:n-7*k_n-2*l).*...
    diff2knY(1+4*k_n+l:n-3*k_n-l));
%%
S1=sum(diffknY(6*k_n:n-5*k_n-2*l).*...
    diffknY(10*k_n+2*l:n-k_n).*...
    diff2knY(8*k_n+2*l:n-3*k_n).*...
    diff2knY(8*k_n+l:n-3*k_n-l))+...
    sum(diffknY(1+3*k_n+l:n-k_n).*...
    diffknY(1+3*k_n:n-k_n-l).*...
    diff2knY(1+k_n+l:n-3*k_n).*...
    diff3knY(1:n-4*k_n-l)) +  U4;
for k = 1:i_n
    %Rk111 = Y(6*k_n+2*k:n-5*k_n-k-2*l)-Y(7*k_n+2*k:n-4*k_n-k-2*l);
    %Rk112 = Y(6*k_n+k:n-5*k_n-2*k-2*l)-Y(4*k_n+k:n-7*k_n-2*k-2*l);
    %Rk121 = Y(10*k_n+2*l+3*k:n-k_n)-Y(11*k_n+2*l+3*k:n);
    %Rk122 = Y(10*k_n+l+2*k:n-k_n-k-l)-Y(8*k_n+l+2*k:n-3k_n-k-l);
    Uk1 = sum(diffknY(6*k_n+2*k:n-5*k_n-k-2*l).*...
    diffknY(10*k_n+2*l+3*k:n-k_n).*...
    diff2knY(8*k_n+2*l+3*k:n-3*k_n).*...
    diff2knY(8*k_n+l+2*k:n-3*k_n-k-l));
%
    %Rk211 = Y(1+3*k_n+l+k:n-k_n)-Y(1+4*k_n+l+k:n);
    %Rk221 = Y(1+3*k_n+k:n-k_n-l)-Y(1+2*k_n+k:n-2*k_n-l);
    %Rk222 = Y(1+3*k_n+l:n-k_n-k)-Y(1+k_n+l:n-3*k_n-k);
    %Rk223 = Y(1+3*k_n:n-k_n-k-l)-Y(1:n-4*k_n-k-l);
    Uk2 = sum(diffknY(1+3*k_n+l+k:n-k_n).*...
    diffknY(1+3*k_n+k:n-k_n-l).*...
    diff2knY(1+k_n+l:n-3*k_n-k).*...
    diff3knY(1:n-4*k_n-k-l));
%
    S1 = S1 + 2*(3*Uk1+Uk2+U4);
end
S2 = U3;
Rj = Rj/n;
S3 = Rj^2*U1 - 2*Rj*U2;
avar = (S1+S2+S3)/n;
avar = max(-avar,avar);
stats.avar = avar;
stats.Rj = Rj;
%
end