%% Estimating the autocorrelation of noise under irregular obv scheme and 
% random noise scales
%% Estimate the autocovariance of noise using ReMeDI and LA
% Author: Z. Merrick Li and Oliver Linton
%%
clear all; clc; clf;
rng(1);
%%
% set parameters to simulate log-prices
simuPar.NumObv = 23400;
simuPar.kappa1 = 0.5;
simuPar.kappa2 = 5/252;
simuPar.mu1 = 3.6;
simuPar.mu2 = 0.04/252;
simuPar.gamma = 0.05/252;
simuPar.varrho =  -0.5;
simuPar.rhoG = 10;
simuPar.sigmaG = 0.1;
simuPar.rho = 0.7;
simuPar.lambda = 1;
simuPar.pricejump = 1;
simuPar.voljump = 1;
simuPar.gc = 5e-4;
knReMeDI = 10;
covorder = 20;
covorders = 0:covorder;
knLA=6;
NumSimu = 1000;
Truecor = (simuPar.rho).^(0:covorder);
ReMeDIcors = [];
LAcors = [];
 parfor i = 1:NumSimu 
       simuData = DGP_irregular(simuPar);
       covsReMeDI = ReMeDI2(simuData.y, 0:covorder, knReMeDI);
       corstemp = covsReMeDI/covsReMeDI(1);
       ReMeDIcors = [ReMeDIcors, corstemp];
       LA = LA2(simuData.y, knLA, covorder+1);
       covsLA = LA.cov;
       LAcors=[LAcors,covsLA/covsLA(1)];
 end
subplot(1,2,1)
plot(covorders,mean(ReMeDIcors'));
hold on;
CIU = mean(ReMeDIcors') + 1.96*std(ReMeDIcors');
CIL = mean(ReMeDIcors') - 1.96*std(ReMeDIcors');
[ph,msg]=jbfill(covorders,CIU,CIL,...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
plot(covorders,Truecor,'r*');
legend('ReMeDI estimates','95% Confidence interval','True autocorrelations');
hold on;
set(gca,'box','off');
subplot(1,2,2)
hold on;
plot(covorders,mean(LAcors'));
CIU = mean(LAcors') + 1.96*std(LAcors');
CIL = mean(LAcors') - 1.96*std(LAcors');
[ph,msg]=jbfill(covorders,CIU,CIL,...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
plot(covorders,Truecor,'r*');
legend('LA estimates','95% Confidence interval','True autocorrelations');
set(gca,'box','off');

