%% Estimate the autocovariance of noise using ReMeDI and LA
% Author: Z. Merrick Li and Oliver Linton
%%
clear all; clc; clf; rng(1);
%%
% set parameters to simulate log-prices
simuPar.kappa1 = 0.5;
simuPar.kappa2 = 5/252;
simuPar.mu1 = 3.6;
simuPar.mu2 = 0.04/252;
simuPar.gamma = 0.05/252;
simuPar.varrho =  -0.5;
simuPar.gc = 3e-4;
simuPar.rho = -0.7;
simuPar.lambda = 1;
fre = 1;% num of obvs per second 
simuPar.delta = 1/(6.5*3600*fre); % 6.5 trading hours
simuPar.T = 1;
knReMeDI = 10;
covorder = 20;
covorders = 0:covorder;
knLA=6;
NumSimu = 1000;
Truecov = (simuPar.gc)^2*(simuPar.rho).^(0:covorder);
obvprices = [];
effiprices = [];   
simuPar.pricejump = 1;
simuPar.voljump = 1; 
ReMeDIcovs = [];
LAcovsADJ = [];
LAcovs = [];
 parfor i = 1:NumSimu 
       [y, X, noise] = DGP_regular(simuPar);
       obvprices =[obvprices,y];
       effiprices = [effiprices,X.logp];
       QV = X.QV;
       covstemp = ReMeDI2(y, 0:covorder, knReMeDI);
       ReMeDIcovs = [ReMeDIcovs, covstemp];
       LA = LA2(y, knLA, covorder+1,QV);
       LAcovs=[LAcovs,LA.cov];
       LAcovsADJ =[LAcovsADJ,LA.cov_adj];
 end
subplot(3,1,1)
plot(covorders,mean(ReMeDIcovs'));
hold on;
CIU = mean(ReMeDIcovs') + 1.96*std(ReMeDIcovs');
CIL = mean(ReMeDIcovs') - 1.96*std(ReMeDIcovs');
[ph,msg]=jbfill(covorders,CIU,CIL,...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
plot(covorders,Truecov,'r*');
ylim([-1e-7,6e-7]);
legend('ReMeDI estimates','95% Confidence interval','True autocovariances');
hold on;
set(gca,'box','off');
subplot(3,1,2)
hold on;
plot(covorders,mean(LAcovs'));
%legend('LA estimates');
CIU = mean(LAcovs') + 1.96*std(LAcovs');
CIL = mean(LAcovs') - 1.96*std(LAcovs');
[ph,msg]=jbfill(covorders,CIU,CIL,...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
plot(covorders,Truecov,'r*');
ylim([-1e-7,6e-7]);
legend('LA estimates','95% Confidence interval','True autocovariances');
set(gca,'box','off');
subplot(3,1,3);
hold on;
plot(covorders,mean(LAcovsADJ'));
CIU = mean(LAcovsADJ') + 1.96*std(LAcovsADJ');
CIL = mean(LAcovsADJ') - 1.96*std(LAcovsADJ');
[ph,msg]=jbfill(covorders,CIU,CIL,...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
plot(covorders,Truecov,'r*');
ylim([-1e-7,6e-7]);
legend('Bias-corrected LA estimates','95% Confidence interval','True autocovariances');
set(gca,'box','off');