%%Compare ReMeDI and LA on different sample sizes 
% and noise-to-signal ratio
% Author: Z. Merrick Li and Oliver Linton
%%
clear all; clc; clf; rng(1);
%%
% set parameters to simulate log-prices
simuPar.kappa1 = 0.5;
simuPar.kappa2 = 5/252;
simuPar.mu1 = 2.6;
simuPar.mu2 = 0.04/252;
simuPar.gamma = 0.05/252;
simuPar.varrho =  -0.5;
simuPar.ga = 0;
simuPar.gb =0;
simuPar.gc = 5e-4;
simuPar.rho = 0.7;
simuPar.lambda = 0;
fre = 1;% num of obvs per second 
Ts=[1,5,20];% samples sizes
NumSamplesizes = length(Ts);
knReMeDI = 10;
covorder = 20;
covorders = 0:covorder;
knLA=6;
NumSimu = 1000;
Truecov = (simuPar.gc)^2*(simuPar.rho).^(0:covorder);
obvprices = [];
effiprices = [];   
simuPar.pricejump = 1;
simuPar.voljump = 0;
simuPar.delta = 1/(6.5*3600*fre);
ReMeDIcovs = zeros(NumSimu,covorder+1,...
    NumSamplesizes);
ReMeDIMSE = zeros(covorder+1,...
    NumSamplesizes);
ReMeDIBias = zeros(covorder+1,...
    NumSamplesizes);
LAcovs = zeros(NumSimu,covorder+1,...
    NumSamplesizes);
LAMSE = zeros(covorder+1,...
    NumSamplesizes);
LABias = zeros(covorder+1,...
    NumSamplesizes);
for j = 1:NumSamplesizes
    simuPar.T = Ts(j);
    ReMeDIcovsvec = [];
    LAcovsvec= [];
    parfor i = 1:NumSimu
        [y, ~, ~] = DGP_regular(simuPar);
        %QV = X.QV;
        ReMeDIcovstemp = ReMeDI2(y, 0:covorder, knReMeDI);
        ReMeDIcovs(i,:,j)=ReMeDIcovstemp';
        ReMeDIcovsvec = [ReMeDIcovsvec, ReMeDIcovstemp];
        LA = LA2(y, knLA, covorder+1);
        LAcovstemp = LA.cov;
        LAcovs(i,:,j) = LAcovstemp';
        LAcovsvec=[LAcovsvec,LA.cov];
    end
    ReMeDIBiastemp = mean(ReMeDIcovsvec') - Truecov;
    ReMeDIVar = (std(ReMeDIcovsvec')).^2;
    ReMeDIMSE(:,j) = sqrt(ReMeDIBiastemp.^2 + ...
        ReMeDIVar);
    ReMeDIBias(:,j) = ReMeDIBiastemp;
    LABiastemp = mean(LAcovsvec') - Truecov;
    LAVar = (std(LAcovsvec')).^2;
    LAMSE(:,j) = sqrt(LABiastemp.^2 + ...
        LAVar);
    LABias(:,j) = LABiastemp;
end
%% Noise-to-signal ratio (NSR)
NSRs = [1e-4, 5e-4, 1e-3];
NumNSRs = length(NSRs);
simuPar.rho = 0.7;
simuPar.lambda = 1;
simuPar.T = 1;
knLA=6; 
ReMeDIcovs = zeros(NumSimu,covorder+1,...
    NumNSRs);
ReMeDIMSENSR = zeros(covorder+1,...
    NumNSRs);
ReMeDIBiasNSR = zeros(covorder+1,...
    NumNSRs);
LAcovs = zeros(NumSimu,covorder+1,...
    NumNSRs);
LAMSENSR = zeros(covorder+1,...
    NumNSRs);
LABiasNSR = zeros(covorder+1,...
    NumNSRs);
for j = 1:NumNSRs
    simuPar.gc = NSRs(j);
    Truecov = (simuPar.gc)^2*(simuPar.rho).^(0:covorder);
    ReMeDIcovsvec = [];
    LAcovsvec= [];
    parfor i = 1:NumSimu
        [y, ~, ~] = DGP_regular(simuPar);
        %QV = X.QV;
        ReMeDIcovstemp = ReMeDI2(y, 0:covorder, knReMeDI);
        ReMeDIcovs(i,:,j)=ReMeDIcovstemp';
        ReMeDIcovsvec = [ReMeDIcovsvec, ReMeDIcovstemp];
        LA = LA2(y, knLA, covorder+1);
        LAcovstemp = LA.cov;
        LAcovs(i,:,j) = LAcovstemp';
        LAcovsvec=[LAcovsvec,LA.cov];
    end
    ReMeDIBiastemp = mean(ReMeDIcovsvec') - Truecov;
    ReMeDIVar = (std(ReMeDIcovsvec')).^2;
    ReMeDIMSENSR(:,j) = sqrt(ReMeDIBiastemp.^2 + ...
        ReMeDIVar);
    ReMeDIBiasNSR(:,j) = ReMeDIBiastemp;
    LABiastemp = mean(LAcovsvec') - Truecov;
    LAVar = (std(LAcovsvec')).^2;
    LAMSENSR(:,j) = sqrt(LABiastemp.^2 + ...
        LAVar);
    LABiasNSR(:,j) = LABiastemp;
end

%% Plot the MSEs
for i = 1:3 
subplot(3,2,2*i-1);
plot(0:20,LAMSE(:,i),'r-*');
hold on;
plot(0:20,ReMeDIMSE(:,i),'b-d');
set(gca,'box','off');
ylim([0,1e-6]);
xlabel('Lags');
ylabel('Meas-squared error');
legend('LA','ReMEDI');
title(['Sample size = ' num2str(Ts(i)*23400)]);
end

for i = 1:3 
subplot(3,2,2*i);
plot(0:20,LAMSENSR(:,i),'r-*');
hold on;
plot(0:20,ReMeDIMSENSR(:,i),'b-d');
set(gca,'box','off');
ylim([0,1.6e-7]);
xlabel('Lags');
ylabel('Meas-squared error');
legend('LA','ReMEDI');
title(['\gamma\equiv ' num2str(NSRs(i))]);
end
