%% estimation on a single trading day
% Author: Z. Merrick Li and Oliver Linton
%%
clear all; clc;
load('KO20180102.mat');% based on the transaction prices
% on 02 Jan 2018.
covorder = 20;
n = length(price);
pars.kn = 10;
pars.phi_n = (pars.kn)^(3/5)/n;
pars.in = 5;
ts = time(:,1)*3600 + time(:,2)*60 + time(:,3);
ReMeDIest = zeros(covorder+1,1);
ReMeDIavar = zeros(covorder+1,1);
knLA = 6;
for l = 0:covorder
    pars.l = l;
    stats = AVarReMeDI2(log(price),ts,pars);
    ReMeDIest(l+1) = stats.Rj;
    ReMeDIavar(l+1) = stats.avar;
end
LA = LA2(log(price), knLA,covorder+1);
covLA  = LA.cov;

%% estimation on a trading week
load('KO2018week2price.mat');
LAweek = LA2(log(KO2018week2price), ...
    knLA,covorder+1);
ReMeDIweek = ReMeDI2(log(KO2018week2price),...
    0:covorder,10);

%% estimation on a trading month
load('cKO.mat');
LAmonth = LA2(log(KOprice), ...
    knLA,covorder+1);
ReMeDImonth = ReMeDI2(log(KOprice),...
    0:covorder,10);
%% plot the estimates
clf;
subplot(3,1,1);
plot(0:covorder,covLA,'r--*');
hold on;
plot(0:covorder,ReMeDIest,'b-d');
plot(0:covorder,zeros(21,1),'k--');
CIU = ReMeDIest + 1.96*sqrt(ReMeDIavar/n);
CIL = ReMeDIest - 1.96*sqrt(ReMeDIavar/n);
ylim([-1e-9,1.2e-8]);
[ph,msg]=jbfill(0:covorder,CIU',CIL',...
[0.3,0.1,0.1],[0.3,0.1,0.1],0,0.1);
xlabel('Lags');
ylabel('Autocovariances');
title('Daily Estimates Based on 02 Jan 2018');
legend('LA estimates','ReMeDI estimates',...
    '95% Confidence interval');
set(gca,'box','off');
%
subplot(3,1,2);
plot(0:covorder,LAweek.cov,'r--*');
hold on;
ylim([-1e-9,1.2e-8]);
plot(0:covorder,ReMeDIweek,'b-d');
plot(0:covorder,zeros(21,1),'k--');
title('Weekly Estimates Based on 08 Jan 2018 to 12 Jan 2018');
legend('LA estimates','ReMeDI estimates');
set(gca,'box','off');
%
subplot(3,1,3);
plot(0:covorder,LAmonth.cov,'r--*');
hold on;
xlabel('Lags');
ylabel('Autocovariances');
plot(0:covorder,ReMeDIweek,'b-d');
plot(0:covorder,zeros(21,1),'k--');
title('Monthly Estimates Based on Jan 2018');
legend('LA estimates','ReMeDI estimates');
set(gca,'box','off');